#include <string.h>
#include <netinet/in.h>

#include "netx.h"

/* ---------------------------------------------------------- */
/* Compute our InterNet 'transport-stack' checksum hdr        */
/* ---------------------------------------------------------- */
unsigned short in_cksum( unsigned short *addr, int len ) {

  int nleft = len;
  int sum   = 0;	          /* 32-bit */
  
  unsigned short *w     = addr;
  unsigned short answer = 0;      /* 16-bit */
  
  /* Our algorithm is simple, using 32 bit accumulator (sum), we add
   * sequential 16 bit words to it, and at the end, fold back all the 
   * carry bits from the top 16 bits into the lower 16 bits.
   */
  
  while(nleft > 1) {
    sum   += *w++;
    nleft -= 2;
  }
  
  /* mop up an odd byte, if necessary */
  if (nleft == 1) {
    *(unsigned char *) (&answer) = *(unsigned char *) w;
    sum += answer;
  }
  /* add back carry outs from top 16 bits to low 16 bits      */
  
  sum  = (sum >> 16) + (sum & 0xffff); /* add hi 16 to low 16 */
  sum += (sum >> 16);                  /* add carry           */
  answer = ~sum;                       /* truncate to 16 bits */
  return (answer);
}

/* ---------------------------------------------------------- */
/* Compute checksum hdr 4 our gentle-packet,                  */
/* Note: Headers must be enterly built be4 calling this!!!    */
/* vita'gasy												  */
/* ---------------------------------------------------------- */
unsigned short gen_cksum( char *pkt )
{
  struct iphdr  *ip;
  struct tcphdr *tcp;
  struct udphdr *udp;
  char          *dta;
  
  struct cksum _cksum;            /* 4 checksumming datagram */
  unsigned short cksum = 0;       /* thE famous cksum        */


  ip   = (struct iphdr  *)pkt;
  tcp  = (struct tcphdr *)(pkt + (ip->ihl << 2));
  udp  = (struct udphdr *)(pkt + (ip->ihl << 2));
  
  bzero( (char *)&_cksum, sizeof(struct cksum) );
  _cksum.src_addr    = ip->saddr;
  _cksum.dst_addr    = ip->daddr;
  
  /* -------------------------------------------------------------------- */
  /* Compute an UDP checksum                                              */
  if ( ip->protocol == IPPROTO_UDP /* 17 */ ) {
    udp->check       = 0;               /* Make sure this is zer0...      */
    _cksum.protocol  = IPPROTO_UDP      /* 17 */;
    _cksum.pr_length = udp->len;        /* in net-byte-order, beware...   */
    bcopy( udp, &_cksum.ut.udp, 8 );    /* Copy udphdr 2 our cksum struct */
    
    dta = (char *) (pkt + (ip->ihl<<2) + 8 );      /* Don't forget 2 copy */
    bcopy( dta, _cksum.buf, ntohs(udp->len) - 8 ); /* data 4 cksumming... */
    
    /* ----------------------------------------------------------------- */
    /* 32 is the pre-pure data, ntohs()-8 is the pure data ...           */
    cksum = in_cksum( (unsigned short *)&_cksum, 32 + ntohs(udp->len) - 8 );
    return cksum;
  }
  
  /* -------------------------------------------------------------------- */
  /* Compute an TCP checksum                                              */
  if ( ip->protocol == IPPROTO_TCP /* 6 */ ) {
    tcp->check       = 0;               /* Make sure this is zer0...      */
    _cksum.protocol  = IPPROTO_TCP      /* 6 */;
    _cksum.pr_length = htons( ntohs(ip->tot_len) - (ip->ihl<<2) );
    bcopy( tcp, &_cksum.ut.tcp, 20 ); 
    
    dta = (char *) (pkt + (ip->ihl<<2) + (tcp->doff<<2)  ); /* Don't forget 2 copy */
    /* ------------------------------------------------------------------- */
    /* Pure data is ip->tot_len - ihl - tcphdr                             */
    bcopy( dta, _cksum.buf, ntohs(ip->tot_len) - (ip->ihl<<2) - (tcp->doff<<2) );
    /* ------------------------------------------------------------------- */
    /* There was a simplification...                                       */
    cksum = in_cksum( (unsigned short *)&_cksum, 12 + ntohs(ip->tot_len) - (ip->ihl<<2) );
    return cksum;
  }
  return 0xffff;  
}
